@@ -55,11 +55,14 @@
 #include "gedit-message-area.h"
 #include "cheese-no-camera.h"
 #include "cheese-prefs-dialog.h"
+#include "cheese-flash.h"
 
 #define SHUTTER_SOUNDS             5
 #define FULLSCREEN_POPUP_HEIGHT    40
 #define FULLSCREEN_TIMEOUT         5 * 1000
 #define FULLSCREEN_EFFECTS_TIMEOUT 15
+#define DEFAULT_WINDOW_WIDTH 600
+#define DEFAULT_WINDOW_HEIGHT 450
 
 typedef enum
 {
@@ -87,7 +90,6 @@ typedef struct
   /* UDI device requested on the command line */
   char *startup_hal_dev_udi;
   char *video_filename;
-  char *photo_filename;
 
   int counter;
 
@@ -172,6 +174,8 @@ typedef struct
   GSource *fullscreen_timeout_source;
 
   int audio_play_counter;
+
+  CheeseFlash *flash;
 } CheeseWindow;
 
 void
@@ -405,15 +409,13 @@ static void
 cheese_window_photo_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
 {
   gdk_threads_enter ();
-  /* TODO look at this g_free */
-  g_free (cheese_window->photo_filename);
-  cheese_window->photo_filename = NULL;
   gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
   gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, TRUE);
   gdk_flush ();
   gdk_threads_leave ();
 }
 
+
 static void
 cheese_window_video_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
 {
@@ -476,7 +478,9 @@ cheese_window_cmd_open (GtkWidget *widget, CheeseWindow *cheese_window)
     dialog = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                      GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-                                     _("Failed to launch program to show:\n%s"), uri);
+                                     _("Failed to launch program to show:\n"
+                                       "%s\n"
+                                       "%s"), uri, error->message);
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
     g_error_free (error);
@@ -1190,20 +1194,23 @@ cheese_window_countdown_picture_cb (gpointer data)
   CheeseWindow *cheese_window = (CheeseWindow *) data;
   GError       *error         = NULL;
   GstAudioPlay *audio_play;
-  char         *file;
+  char         *shutter_filename;
+  char         *photo_filename;
 
-  file       = audio_play_get_filename (cheese_window);
-  audio_play = gst_audio_play_file (file, &error);
-  if (!audio_play)
+  photo_filename = cheese_fileutil_get_new_media_filename (cheese_window->fileutil, WEBCAM_MODE_PHOTO);
+  if (cheese_webcam_take_photo (cheese_window->webcam, photo_filename))
   {
-    g_warning ("%s", error ? error->message : "Unknown error");
-    g_error_free (error);
+    cheese_flash_fire (cheese_window->flash);
+    shutter_filename = audio_play_get_filename (cheese_window);
+    audio_play = gst_audio_play_file (shutter_filename, &error);
+    if (!audio_play)
+    {
+      g_warning ("%s", error ? error->message : "Unknown error");
+      g_error_free (error);
+    }
+    g_free (shutter_filename);
   }
-
-  g_free (file);
-
-  cheese_window->photo_filename = cheese_fileutil_get_new_media_filename (cheese_window->fileutil, WEBCAM_MODE_PHOTO);
-  cheese_webcam_take_photo (cheese_window->webcam, cheese_window->photo_filename);
+  g_free (photo_filename);
 }
 
 static void
@@ -1656,6 +1663,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->thumb_view = cheese_thumb_view_new ();
   cheese_window->thumb_nav  = eog_thumb_nav_new (cheese_window->thumb_view, TRUE);
   gtk_container_add (GTK_CONTAINER (cheese_window->thumb_scrollwindow), cheese_window->thumb_nav);
+  /* show the scroll window to get it included in the size requisition done later */
+  gtk_widget_show_all (cheese_window->thumb_scrollwindow);
 
   char *gconf_effects;
   g_object_get (cheese_window->gconf, "gconf_prop_selected_effects", &gconf_effects, NULL);
@@ -1675,6 +1684,10 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->countdown_fullscreen = cheese_countdown_new ();
   gtk_container_add (GTK_CONTAINER (cheese_window->countdown_frame_fullscreen), cheese_window->countdown_fullscreen);
 
+  gtk_widget_realize (cheese_window->screen);
+  gdk_window_set_back_pixmap (cheese_window->screen->window, NULL, FALSE);
+  gtk_widget_set_app_paintable (cheese_window->screen, TRUE);
+  gtk_widget_set_double_buffered (cheese_window->screen, FALSE);
   gtk_widget_add_events (cheese_window->screen, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
 
   cheese_window->ui_manager = gtk_ui_manager_new ();
@@ -1942,6 +1955,7 @@ setup_camera (CheeseWindow *cheese_window)
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture), TRUE);
   gtk_widget_set_sensitive (GTK_WIDGET (cheese_window->take_picture_fullscreen), TRUE);
   gtk_action_group_set_sensitive (cheese_window->actions_effects, TRUE);
+
   gdk_threads_leave ();
 }
 
@@ -1954,8 +1968,9 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
 
   cheese_window->startup_hal_dev_udi = hal_dev_udi;
   cheese_window->gconf               = cheese_gconf_new ();
-  cheese_window->audio_play_counter  = 0;
   cheese_window->fileutil            = cheese_fileutil_new ();
+  cheese_window->flash               = cheese_flash_new ();
+  cheese_window->audio_play_counter  = 0;
   cheese_window->isFullscreen        = FALSE;
 
   cheese_window->server = dbus_server;
@@ -1968,7 +1983,6 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
   cheese_window_create_window (cheese_window);
   gtk_action_group_set_sensitive (cheese_window->actions_effects, FALSE);
 
-  gtk_widget_show_all (cheese_window->window);
   ephy_spinner_start (EPHY_SPINNER (cheese_window->throbber));
 
   gtk_notebook_set_current_page (GTK_NOTEBOOK (cheese_window->notebook), 2);
@@ -1976,6 +1990,17 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
   cheese_window->webcam_mode = WEBCAM_MODE_PHOTO;
   cheese_window->recording   = FALSE;
 
+  /* handy trick to set default size of the drawing area while not
+   * limiting its minimum size, thanks Owen! */
+  GtkRequisition req;
+  gtk_widget_set_size_request (cheese_window->screen,
+                               DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT);
+  gtk_widget_size_request (cheese_window->window, &req);
+  gtk_window_set_default_size (GTK_WINDOW (cheese_window->window), req.width, req.height);
+  gtk_widget_set_size_request (cheese_window->screen, -1, -1);
+
+  gtk_widget_show_all (cheese_window->window);
+
   /* Run cam setup in its own thread */
   GError *error = NULL;
   if (!g_thread_create ((GThreadFunc) setup_camera, cheese_window, FALSE, &error))
